
import bmcs_utils.api as bu
import sympy as sp


class CB_ELF_ELM_Symb(bu.SymbExpr):

    E_m, A_m = sp.symbols(r'E_\mathrm{m}, A_\mathrm{m}', nonnegative=True)
    E_f, A_f = sp.symbols(r'E_\mathrm{f}, A_\mathrm{f}', nonnegative=True)
    tau, p = sp.symbols(r'\bar{\tau}, p', nonnegative=True)
    C, D, E, F = sp.symbols('C, D, E, F')
    P, w = sp.symbols('P, w')
    x, a, L_b = sp.symbols('x, a, L_b')

    d_sig_f = p * tau / A_f
    d_sig_m = -p * tau / A_m

    sig_f = sp.integrate(d_sig_f, x) + C
    sig_m = sp.integrate(d_sig_m, x) + D

    eps_f = sig_f / E_f
    eps_m = sig_m / E_m

    u_f = sp.integrate(eps_f, x) + E
    u_m = sp.integrate(eps_m, x) + F

    eq_C = {P - sig_f.subs({x:0}) * A_f}
    C_subs = sp.solve(eq_C,C)
    eq_D = {sig_m.subs({x:0}) * A_m}
    D_subs = sp.solve(eq_D,D)

    eqns_u_equal = {u_f.subs(C_subs).subs(x,a) - u_m.subs(D_subs).subs(x,a)}
    E_subs = sp.solve(eqns_u_equal,E)

    eqns_eps_equal = {eps_f.subs(C_subs).subs(x,a) - eps_m.subs(D_subs).subs(x,a)}

    a_subs = sp.solve(eqns_eps_equal,a)

    var_subs = {**C_subs,**D_subs,**E_subs,**a_subs}

    u_f_x = sp.simplify( u_f.subs(var_subs) )
    u_m_x = sp.simplify( u_m.subs(var_subs) )

    A_c = A_m + A_f
    E_c = (E_m * A_m + E_f * A_f) / (A_c)
    G = sp.symbols('G')
    u_c_x_ = P / A_c / E_c * x + G
    G_subs = sp.solve( u_c_x_.subs(x, -L_b), G)[0]

    u_c_x = sp.simplify(u_c_x_.subs(G, G_subs).subs(var_subs))
    u_c_a = sp.simplify( u_c_x.subs(x,var_subs[a]) )

    F_sol = sp.simplify( sp.solve( u_m_x.subs(x, a_subs[a]) - u_c_a, F)[0] )
    u_mc_x = u_m_x.subs(F, F_sol)
    u_fc_x = u_f_x.subs(F, F_sol)

    u_fa_x = sp.Piecewise((u_c_x, x <= var_subs[a]),
                          (u_fc_x, x > var_subs[a])
                         )
    u_ma_x = sp.Piecewise((u_c_x, x <= var_subs[a]),
                          (u_mc_x, x > var_subs[a]),
                         )
    eps_f_x = sp.diff(u_fa_x,x)
    eps_m_x = sp.diff(u_ma_x,x)

    sig_f_x = E_f * eps_f_x
    sig_m_x = E_m * eps_m_x

    tau_x = sig_f_x.diff(x) * A_f / p

    eps_f_0 = P / E_f / A_f
    eps_m_0 = -P / E_m / A_m
    a_subs = sp.solve({P - p * tau * a}, a)
    w_el = sp.Rational(1,2) * ( eps_f_0 - eps_m_0) * a
    Pw_pull_ = sp.solve(w_el.subs(a_subs) - w, P)[1]

    w_L_b = u_fa_x.subs(x, -L_b).subs(P, Pw_pull_)

    aw_pull = a_subs[a].subs(P, Pw_pull_)

    P_max = p * tau * L_b
    w_argmax = sp.solve(P_max - Pw_pull_, w)[0]
    d_Pw_pull_dw = sp.diff(Pw_pull_, w)
    K_c = sp.simplify(d_Pw_pull_dw.subs({w: w_argmax}))

    Pw_clamped = sp.Piecewise(
        (Pw_pull_, w < w_argmax),
        (P_max + K_c * (w - w_argmax), w >= w_argmax)
    )

    Pw_pull = Pw_clamped

    #-------------------------------------------------------------------------
    # Declaration of the lambdified methods
    #-------------------------------------------------------------------------

    symb_model_params = ['E_f', 'A_f', 'E_m', 'A_m', 'tau', 'p', 'L_b']

    symb_expressions = [
        ('eps_f_x', ('x','P',)),
        ('eps_m_x', ('x','P',)),
        ('sig_f_x', ('x','P',)),
        ('sig_m_x', ('x','P',)),
        ('tau_x', ('x','P',)),
        ('u_fa_x', ('x','P',)),
        ('u_ma_x', ('x','P',)),
        ('w_L_b', ('w',)),
        ('aw_pull', ('w',)),
        ('Pw_pull', ('w',)),
    ]
